import 'dart:math';

import 'package:clean_android_design/clean_android_design.dart';
import 'package:flutter/material.dart';

import 'i_help_widget.dart';

class HelpSideWidget extends StatefulWidget {
  final IHelpWidget helpWidget;

  const HelpSideWidget({
    Key key,
    @required this.helpWidget,
  })  : assert(helpWidget != null),
        super(key: key);

  @override
  _HelpSideWidgetState createState() => _HelpSideWidgetState();
}

class _HelpSideWidgetState extends State<HelpSideWidget>
    with SingleTickerProviderStateMixin {
  AnimationController _animationController;
  Animation _widthTween;
  Animation _paddingTween;
  Animation _whiteColorTween;
  Animation _fontTween;

  bool _waitForOpenNavigator = false;
  int _waitForAnimationId;

  @override
  void initState() {
    super.initState();

    _animationController = AnimationController(
      vsync: this,
      duration: Duration(milliseconds: 300),
      reverseDuration: Duration(milliseconds: 100),
    );

    var curvedAnimation =
        CurvedAnimation(parent: _animationController, curve: Curves.ease);

    _widthTween = IntTween(begin: 85, end: 30).animate(curvedAnimation);
    _whiteColorTween =
        ColorTween(begin: Colors.white, end: Colors.white.withAlpha(0))
            .animate(curvedAnimation);
    _fontTween = TextStyleTween(
        begin: TextStyles.defaultTextStyle().copyWith(
          color: CleanAndroidDesignDefaultColors.accent,
          fontWeight: FontWeight.bold,
          fontSize: 15,
        ),
        end: TextStyles.defaultTextStyle().copyWith(
          color: Colors.white,
          fontWeight: FontWeight.bold,
          fontSize: 22,
        )).animate(curvedAnimation);
    _paddingTween = EdgeInsetsTween(
            begin: EdgeInsets.all(8.0),
            end: EdgeInsets.only(bottom: 6.0, left: 4.0))
        .animate(curvedAnimation);

    _animationController.addStatusListener((status) {
      if (status == AnimationStatus.dismissed)
        _closeInDuration(Duration(milliseconds: 3000));
    });
  }

  _closeInDuration(Duration duration) {
    var id = Random().nextInt(100000);
    _waitForAnimationId = id;
    Future.delayed(duration, () {
      if (_animationController.status == AnimationStatus.dismissed &&
          !_waitForOpenNavigator &&
          _waitForAnimationId == id &&
          mounted) {
        _animationController.forward();
      }
    });
  }

  @override
  Widget build(BuildContext context) {
    if (!widget.helpWidget.showHelp()) return Container();

    var isNew = widget.helpWidget.isNew();
    if (!isNew) {
      _animationController.animateTo(1, duration: Duration(milliseconds: 0));
    } else {
      _closeInDuration(Duration(milliseconds: 2000));
    }

    var padding = MediaQuery.of(context).size.height / 3 * 2;

    return Align(
      alignment: Alignment.topRight,
      child: Padding(
        padding: EdgeInsets.only(top: padding),
        child: Hero(
          tag: "HelpIcon",
          child: Material(
            color: CleanAndroidDesignDefaultColors.accent,
            borderRadius: BorderRadius.only(
                topLeft: Radius.circular(8), bottomLeft: Radius.circular(8)),
            child: InkWell(
              onTap: () {
                if (_animationController.status == AnimationStatus.completed)
                  _animationController.reverse();
                else
                  _showHelpWidget();
              },
              borderRadius: BorderRadius.only(
                  topLeft: Radius.circular(8), bottomLeft: Radius.circular(8)),
              highlightColor: CleanAndroidDesignDefaultColors.accentLight,
              child: AnimatedBuilder(
                animation: _widthTween,
                child: Container(),
                builder: (context, _) => Container(
                  height: 45,
                  width: _widthTween.value + 0.0,
                  decoration: BoxDecoration(
                    borderRadius: BorderRadius.only(
                        topLeft: Radius.circular(8),
                        bottomLeft: Radius.circular(8)),
                    border: Border.all(
                        color:
                            CleanAndroidDesignDefaultColors.defaultColorLight),
                  ),
                  child: Row(
                    children: [
                      Padding(
                        padding: _paddingTween.value,
                        child: Container(
                          height: 20,
                          width: 20,
                          decoration: BoxDecoration(
                            color: _whiteColorTween.value,
                            borderRadius: BorderRadius.circular(4),
                          ),
                          child: Center(
                            child: Text(
                              "?",
                              style: _fontTween.value,
                            ),
                          ),
                        ),
                      ),
                      !_widthTween.isCompleted
                          ? Text(
                              isNew ? "Neu" : "Hilfe",
                              style: TextStyles.defaultTextStyle().copyWith(
                                color: Colors.white,
                                fontSize: 16,
                                fontWeight: FontWeight.bold,
                              ),
                            )
                          : Container(),
                    ],
                  ),
                ),
              ),
            ),
          ),
        ),
      ),
    );
  }

  _showHelpWidget() async {
    _waitForOpenNavigator = true;
    _animationController.reverse();

    await SplitNavigator.of(SplitNavigatorSide.right).push(MaterialPageRoute(
      builder: (context) => widget.helpWidget.buildHelpWidget(context),
    ));
    _waitForOpenNavigator = false;
    _closeInDuration(Duration(milliseconds: 400));
  }
}
